@@ -1,16 +1,25 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
+from __future__ import with_statement
+
 import os
 import sys
 import gettext
-import urlparse
+from urlparse import urlparse
+import urllib
 import gobject
 import gtk
+if sys.version_info < (2, 6):
+    import simplejson as json
+else:
+    import json
+import webbrowser
 import tryton.rpc as rpc
 from tryton.config import CONFIG, TRYTON_ICON, PIXMAPS_DIR, DATA_DIR, \
         get_config_dir
 import tryton.common as common
 from tryton.action import Action
+from tryton.wizard import Wizard
 from tryton.gui.window import Window
 from tryton.gui.window.preference import Preference
 from tryton.gui.window import Limit
@@ -29,6 +38,7 @@ import tryton.translate as translate
 import tryton.plugins
 import pango
 import time
+import threading
 try:
     import gtk_osxapplication
 except ImportError:
@@ -45,13 +55,17 @@ _MAIN = []
 
 class Main(object):
     window = None
+    tryton_client = None
 
-    def __init__(self):
+    def __init__(self, tryton_client):
         super(Main, self).__init__()
+        Main.tryton_client = tryton_client
 
         self.window = gtk.Window()
         self._width = int(CONFIG['client.default_width'])
         self._height = int(CONFIG['client.default_height'])
+        if CONFIG['client.maximize']:
+            self.window.maximize()
         self.window.set_default_size(self._width, self._height)
         self.window.set_resizable(True)
         self.window.set_title('Tryton')
@@ -59,6 +73,7 @@ class Main(object):
         self.window.connect("destroy", Main.sig_quit)
         self.window.connect("delete_event", self.sig_delete)
         self.window.connect('configure_event', self.sig_configure)
+        self.window.connect('window_state_event', self.sig_window_state)
 
         self.accel_group = gtk.AccelGroup()
         self.window.add_accel_group(self.accel_group)
@@ -72,6 +87,13 @@ class Main(object):
                 gtk.gdk.CONTROL_MASK)
         gtk.accel_map_add_entry('<tryton>/File/Quit', gtk.keysyms.Q,
                 gtk.gdk.CONTROL_MASK)
+        gtk.accel_map_add_entry('<tryton>/User/Menu Reload', gtk.keysyms.T,
+                gtk.gdk.MOD1_MASK)
+        gtk.accel_map_add_entry('<tryton>/User/Menu Toggle', gtk.keysyms.T,
+                gtk.gdk.CONTROL_MASK)
+        gtk.accel_map_add_entry('<tryton>/User/Home', gtk.keysyms.H,
+                gtk.gdk.CONTROL_MASK)
+
         gtk.accel_map_add_entry('<tryton>/Form/New', gtk.keysyms.N,
                 gtk.gdk.CONTROL_MASK)
         gtk.accel_map_add_entry('<tryton>/Form/Save', gtk.keysyms.S,
@@ -88,10 +110,6 @@ class Main(object):
                 0)
         gtk.accel_map_add_entry('<tryton>/Form/Switch View', gtk.keysyms.L,
                 gtk.gdk.CONTROL_MASK)
-        gtk.accel_map_add_entry('<tryton>/Form/Menu', gtk.keysyms.T,
-                gtk.gdk.CONTROL_MASK)
-        gtk.accel_map_add_entry('<tryton>/Form/Home', gtk.keysyms.H,
-                gtk.gdk.CONTROL_MASK)
         gtk.accel_map_add_entry('<tryton>/Form/Close', gtk.keysyms.W,
                 gtk.gdk.CONTROL_MASK)
         gtk.accel_map_add_entry('<tryton>/Form/Previous Tab', gtk.keysyms.Page_Up,
@@ -116,6 +134,7 @@ class Main(object):
         self.toolbar = toolbar
         toolbar.set_orientation(gtk.ORIENTATION_HORIZONTAL)
         toolbar.set_style(gtk.TOOLBAR_BOTH)
+        self.toolbar_menu = None
 
         self.vbox = gtk.VBox()
         self.window.add(self.vbox)
@@ -123,7 +142,6 @@ class Main(object):
         self.status_hbox = None
         self.menubar = None
         self.menuitem_user = None
-        self.menuitem_form = None
         self.menuitem_plugins = None
 
         self.set_statusbar()
@@ -132,18 +150,20 @@ class Main(object):
         if self.macapp is not None:
             self.macapp.ready()
 
-
-        self.vbox.pack_start(toolbar, False, True)
-
         self.buttons = {}
         self._set_toolbar()
         self.set_toolbar_label()
 
+        self.pane = gtk.HPaned()
+        self.menu_screen = None
+        self.vbox.pack_start(self.pane, True, True)
+
         self.notebook = gtk.Notebook()
         self.notebook.popup_enable()
         self.notebook.set_scrollable(True)
         self.notebook.connect_after('switch-page', self._sig_page_changt)
-        self.vbox.pack_start(self.notebook, True, True)
+
+        self.pane.add2(self.notebook)
 
         self.window.show_all()
 
@@ -210,20 +230,6 @@ class Main(object):
         menu_user.set_accel_group(self.accel_group)
         menu_user.set_accel_path('<tryton>/User')
 
-        menuitem_form = gtk.MenuItem(_('For_m'))
-        if self.menuitem_form:
-            menuitem_form.set_sensitive(
-                    self.menuitem_form.get_property('sensitive'))
-        else:
-            menuitem_form.set_sensitive(False)
-        self.menuitem_form = menuitem_form
-        menubar.add(menuitem_form)
-
-        menu_form = self._set_menu_form()
-        menuitem_form.set_submenu(menu_form)
-        menu_form.set_accel_group(self.accel_group)
-        menu_form.set_accel_path('<tryton>/Form')
-
         menuitem_options = gtk.MenuItem(_('_Options'))
         menubar.add(menuitem_options)
 
@@ -266,7 +272,6 @@ class Main(object):
             self.macapp.insert_app_menu_item(self.aboutitem,0)
             menuitem_file.show_all()
             menuitem_user.show_all()
-            menuitem_form.show_all()
             menuitem_options.show_all()
             menuitem_plugins.show_all()
             menuitem_shortcut.show_all()
@@ -392,6 +397,31 @@ class Main(object):
 
         menu_user.add(gtk.SeparatorMenuItem())
 
+        imagemenuitem_menu = gtk.ImageMenuItem(_('_Menu Reload'), self.accel_group)
+        image = gtk.Image()
+        image.set_from_stock('tryton-start-here', gtk.ICON_SIZE_MENU)
+        imagemenuitem_menu.set_image(image)
+        imagemenuitem_menu.connect('activate', self.sig_win_menu)
+        imagemenuitem_menu.set_accel_path('<tryton>/User/Menu Reload')
+        menu_user.add(imagemenuitem_menu)
+
+        imagemenuitem_menu_toggle = gtk.ImageMenuItem(_('_Menu Toggle'),
+                self.accel_group)
+        imagemenuitem_menu_toggle.connect('activate', lambda *a: self.menu_toggle())
+        imagemenuitem_menu_toggle.set_accel_path('<tryton>/User/Menu Toggle')
+        menu_user.add(imagemenuitem_menu_toggle)
+
+
+        imagemenuitem_home = gtk.ImageMenuItem(_('_Home'), self.accel_group)
+        image = gtk.Image()
+        image.set_from_stock('tryton-go-home', gtk.ICON_SIZE_MENU)
+        imagemenuitem_home.set_image(image)
+        imagemenuitem_home.connect('activate', self.sig_home_new)
+        imagemenuitem_home.set_accel_path('<tryton>/User/Home')
+        menu_user.add(imagemenuitem_home)
+
+        menu_user.add(gtk.SeparatorMenuItem())
+
         imagemenuitem_send_request = gtk.ImageMenuItem(_('_Send a Request'))
         image = gtk.Image()
         image.set_from_stock('tryton-mail-message-new', gtk.ICON_SIZE_MENU)
@@ -478,24 +508,8 @@ class Main(object):
         imagemenuitem_switch.set_accel_path('<tryton>/Form/Switch View')
         menu_form.add(imagemenuitem_switch)
 
-        imagemenuitem_menu = gtk.ImageMenuItem(_('_Menu'), self.accel_group)
-        image = gtk.Image()
-        image.set_from_stock('tryton-start-here', gtk.ICON_SIZE_MENU)
-        imagemenuitem_menu.set_image(image)
-        imagemenuitem_menu.connect('activate', self.sig_win_menu)
-        imagemenuitem_menu.set_accel_path('<tryton>/Form/Menu')
-        menu_form.add(imagemenuitem_menu)
-
         menu_form.add(gtk.SeparatorMenuItem())
 
-        imagemenuitem_home = gtk.ImageMenuItem(_('_Home'), self.accel_group)
-        image = gtk.Image()
-        image.set_from_stock('tryton-go-home', gtk.ICON_SIZE_MENU)
-        imagemenuitem_home.set_image(image)
-        imagemenuitem_home.connect('activate', self.sig_home_new)
-        imagemenuitem_home.set_accel_path('<tryton>/Form/Home')
-        menu_form.add(imagemenuitem_home)
-
         imagemenuitem_close = gtk.ImageMenuItem(_('_Close Tab'), self.accel_group)
         image = gtk.Image()
         image.set_from_stock('tryton-close', gtk.ICON_SIZE_MENU)
@@ -574,6 +588,10 @@ class Main(object):
                 'but_import')
         menuitem_import.set_accel_path('<tryton>/Form/Import Data')
         menu_form.add(menuitem_import)
+
+        menu_form.set_accel_group(self.accel_group)
+        menu_form.set_accel_path('<tryton>/Form')
+        menu_form.show_all()
         return menu_form
 
     def _set_menu_options(self):
@@ -876,12 +894,6 @@ class Main(object):
         toolbutton_reload.connect('clicked', self._sig_child_call, 'but_reload')
         self.buttons['but_reload'] = toolbutton_reload
 
-        toolbutton_menu = gtk.ToolButton('tryton-start-here')
-        self.toolbutton_menu = toolbutton_menu
-        self.toolbar.insert(toolbutton_menu, -1)
-        toolbutton_menu.connect('clicked', self.sig_win_menu)
-        self.buttons['but_menu'] = toolbutton_menu
-
         self.toolbar.insert(gtk.SeparatorToolItem(), -1)
 
         toolbutton_action = gtk.ToolButton('tryton-executable')
@@ -901,13 +913,18 @@ class Main(object):
         toolbutton_attach.connect('clicked', self._sig_child_call, 'but_attach')
         self.buttons['but_attach'] = toolbutton_attach
 
-        self.toolbar.insert(gtk.SeparatorToolItem(), -1)
+        self.menutoolbutton = None
+        if hasattr(gtk, 'MenuToolButton'):
+            self.menutoolbutton = gtk.MenuToolButton(None, None)
+            # Remove the button to keep only the toggle
+            hbox, = self.menutoolbutton.get_children()
+            button, toggle = hbox.get_children()
+            hbox.remove(button)
+            self.toolbar.insert(self.menutoolbutton, -1)
+            menu = self._set_menu_form()
+            self.menutoolbutton.set_menu(menu)
+            self.toolbar_menu = menu
 
-        toolbutton_request = gtk.ToolButton('tryton-mail-message')
-        self.toolbutton_request = toolbutton_request
-        self.toolbar.insert(toolbutton_request, -1)
-        toolbutton_request.connect('clicked', self.sig_request_open)
-        self.buttons['but_request'] = toolbutton_request
 
     def set_toolbar_label(self):
         labels = {
@@ -919,11 +936,9 @@ class Main(object):
             'but_next': _('Next'),
             'but_switch': _('Switch'),
             'but_reload': _('_Reload'),
-            'but_menu': _('Menu'),
             'but_action': _('Action'),
             'but_print': _('Print'),
             'but_attach': _('Attachment(0)'),
-            'but_request': _('Request'),
         }
         tooltips = {
             'but_new': _('Create a new record'),
@@ -934,15 +949,17 @@ class Main(object):
             'but_next': _('Next Record'),
             'but_switch': _('Switch view'),
             'but_reload': _('Reload'),
-            'but_menu': _('Menu'),
             'but_action': _('Action'),
             'but_print': _('Print'),
             'but_attach': _('Add an attachment to the record'),
-            'but_request': _('Request'),
         }
         for i in self.buttons:
             self.buttons[i].set_label(labels[i])
             self.tooltips.set_tip(self.buttons[i], tooltips[i])
+        if self.toolbar_menu and self.menutoolbutton:
+            self.toolbar_menu.destroy()
+            menu = self._set_menu_form()
+            self.menutoolbutton.set_menu(menu)
 
     @staticmethod
     def get_main():
@@ -955,6 +972,34 @@ class Main(object):
                 'id': action,
                 'ids': [action],
                 })
+
+        def _add_shortcut(widget):
+            ids = self.menu_screen.sel_ids_get()
+            if not ids:
+                return
+            try:
+                values = rpc.execute('model', self.menu_screen.model_name,
+                        'read', ids, ['rec_name'], rpc.CONTEXT)
+            except Exception, exception:
+                common.process_exception(exception, self.window)
+                return
+            try:
+                for value in values:
+                    rpc.execute('model', 'ir.ui.view_sc', 'create', {
+                        'name': value['rec_name'],
+                        'res_id': value['id'],
+                        'user_id': rpc._USER,
+                        'resource': self.menu_screen.model_name,
+                        }, rpc.CONTEXT)
+            except Exception, exception:
+                common.process_exception(exception, self.window)
+            self.shortcut_set()
+
+        def _manage_shortcut(widget):
+            Window.create(False, 'ir.ui.view_sc', False,
+                    domain=[('user_id', '=', rpc._USER)],
+                    window=self.window, mode=['tree', 'form'])
+
         if shortcuts is None:
             user = rpc._USER
             try:
@@ -967,6 +1012,13 @@ class Main(object):
             menuitem = gtk.MenuItem(shortcut['name'])
             menuitem.connect('activate', _action_shortcut, shortcut['res_id'])
             menu.add(menuitem)
+        menu.add(gtk.MenuItem())
+        add_shortcut = gtk.MenuItem(_('Add Shortcut'))
+        add_shortcut.connect('activate', _add_shortcut)
+        menu.add(add_shortcut)
+        manage_shortcut = gtk.MenuItem(_('Manage Shortcut'))
+        manage_shortcut.connect('activate', _manage_shortcut)
+        menu.add(manage_shortcut)
         menu.show_all()
         self.menuitem_shortcut.set_submenu(menu)
         self.menuitem_shortcut.set_sensitive(True)
@@ -1067,22 +1119,22 @@ class Main(object):
                     self.shortcut_set()
                     self.set_statusbar()
                     self.request_set()
-                    self.sig_reload_menu()
+                    self.sig_win_menu()
                 CONFIG['client.lang'] = prefs['language']
             CONFIG.save()
         self.window.present()
         return True
 
     def sig_win_close(self, widget):
-        self._sig_child_call(widget, 'but_close')
+        self._sig_remove_book(widget,
+                self.notebook.get_nth_page(self.notebook.get_current_page()))
 
     def sig_request_new(self, widget):
         ctx = {}
         ctx.update(rpc.CONTEXT)
         ctx['active_test'] = False
-        return Window.create(None, 'res.request', False, [
-            ], 'form', mode=['form', 'tree'], window=self.window,
-            context=ctx)
+        return Window.create(None, 'res.request', False, [ ],
+                mode=['form', 'tree'], window=self.window, context=ctx)
 
     def sig_request_open(self, widget):
         ctx = {}
@@ -1096,9 +1148,8 @@ class Main(object):
             else:
                 raise
         ids = ids1 + ids2
-        return Window.create(False, 'res.request', ids, [
-            ], 'form', mode=['tree', 'form'], window=self.window,
-            context=ctx)
+        return Window.create(False, 'res.request', ids, [ ],
+                mode=['tree', 'form'], window=self.window, context=ctx)
 
     def request_set(self, exception=False):
         try:
@@ -1114,13 +1165,6 @@ class Main(object):
                 ids, ids2 = rpc.execute('model', 'res.request', 'request_get',
                         rpc.CONTEXT)
             label = _('Requests (%s/%s)') % (len(ids), len(ids2))
-            self.buttons['but_request'].set_label(label)
-            self.tooltips.set_tip(self.buttons['but_request'], label)
-            if not ids:
-                self.buttons['but_request'].set_stock_id('tryton-mail-message')
-            else:
-                self.buttons['but_request'].set_stock_id(
-                        'tryton-mail-message-new')
             message = _('Waiting requests: %s received - %s sent') % (len(ids),
                         len(ids2))
             self.sb_requests.set_text(message)
@@ -1130,12 +1174,13 @@ class Main(object):
                 raise
             return ([], [])
 
-    def sig_login(self, widget=None, dbname=False, res=None):
-        self.sig_logout(widget, disconnect=False)
+    def sig_login(self, widget=None, res=None):
+        if not self.sig_logout(widget, disconnect=False):
+            return
         if not res:
             try:
                 dblogin = DBLogin(self.window)
-                res = dblogin.run(dbname, self.window)
+                res = dblogin.run(self.window)
             except Exception, exception:
                 if exception.args == ('QueryCanceled',):
                     return False
@@ -1154,6 +1199,7 @@ class Main(object):
                         False, rpc.CONTEXT)
             except Exception:
                 prefs = None
+            common.ICONFACTORY.load_icons()
             if prefs and 'language_direction' in prefs:
                 translate.set_language_direction(prefs['language_direction'])
                 CONFIG['client.language_direction'] = prefs['language_direction']
@@ -1180,18 +1226,18 @@ class Main(object):
             return self.sig_login()
         if not self.menuitem_shortcut.get_property('sensitive'):
             self.shortcut_set()
-        self.toolbutton_menu.set_sensitive(True)
-        self.toolbutton_request.set_sensitive(True)
         self.menuitem_user.set_sensitive(True)
-        self.menuitem_form.set_sensitive(True)
+        #self.menuitem_form.set_sensitive(True)
         self.menuitem_plugins.set_sensitive(True)
-        self.notebook.grab_focus()
+        if CONFIG.arguments:
+            url = CONFIG.arguments.pop()
+            self.open_url(url)
         return True
 
     def sig_logout(self, widget, disconnect=True):
         res = True
         while res:
-            wid = self._wid_get()
+            wid = self.get_page()
             if wid:
                 if 'but_close' in wid.handlers:
                     res = wid.handlers['but_close']()
@@ -1200,15 +1246,19 @@ class Main(object):
                 res = self._win_del()
             else:
                 res = False
+        if self.pane.get_child1():
+            self.pane.remove(self.pane.get_child1())
+            if self.pane.get_position():
+                CONFIG['menu.pane'] = self.pane.get_position()
+        if self.menu_screen:
+            self.menu_screen.destroy()
+            self.menu_screen = None
         self.sb_username.set_text('')
         self.sb_servername.set_text('')
         self.sb_requests.set_text('')
         self.shortcut_unset()
-        self.toolbutton_menu.set_sensitive(False)
-        self.toolbutton_request.set_sensitive(False)
-        self.toolbutton_request.set_label(_('Request'))
         self.menuitem_user.set_sensitive(False)
-        self.menuitem_form.set_sensitive(False)
+        #self.menuitem_form.set_sensitive(False)
         self.menuitem_plugins.set_sensitive(False)
         if disconnect:
             rpc.logout()
@@ -1236,23 +1286,32 @@ class Main(object):
     def sig_shortcuts(self, widget):
         Shortcuts(self.window).run()
 
-    def sig_reload_menu(self):
-        res = False
-        for page in range(len(self.pages)):
-            if self.pages[page].model == 'ir.ui.menu':
-                self.pages[page].sig_reload()
-                hbox = self.notebook.get_tab_label(self.pages[page].widget)
-                label = hbox.get_children()[0]
-                label.set_text(_('Menu'))
-                res = True
-        return res
+    def menu_toggle(self, nohide=False):
+        has_focus = True
+        if (self.menu_screen
+                and self.menu_screen.current_view.view_type == 'tree'):
+            try:
+                has_focus = self.menu_screen.current_view.widget_tree.has_focus()
+            except AttributeError:
+                has_focus = (self.menu_screen.current_view.widget_tree.flags()
+                        & gtk.HAS_FOCUS)
+        if self.pane.get_position() and has_focus:
+            CONFIG['menu.pane'] = self.pane.get_position()
+            if not nohide:
+                self.pane.set_position(0)
+                self.notebook.grab_focus()
+        else:
+            self.pane.set_position(int(CONFIG['menu.pane']))
+            if self.menu_screen:
+                self.menu_screen.current_view.set_cursor()
 
     def sig_win_menu(self, widget=None, quiet=True, prefs=None):
-        for page in range(len(self.pages)):
-            if self.pages[page].model == 'ir.ui.menu':
-                page_num = self.notebook.page_num(self.pages[page].widget)
-                self.notebook.set_current_page(page_num)
-                return True
+        if self.pane.get_child1():
+            self.pane.remove(self.pane.get_child1())
+            if self.pane.get_position():
+                CONFIG['menu.pane'] = self.pane.get_position()
+        self.menu_screen = None
+        self.menu_toggle(nohide=True)
         res = self.sig_win_new(widget, menu_type='menu', quiet=quiet,
                 prefs=prefs)
         return res
@@ -1292,6 +1351,8 @@ class Main(object):
 
     def sig_plugin_execute(self, widget):
         page = self.notebook.get_current_page()
+        if page == -1:
+            return
         datas = {
                 'model': self.pages[page].model,
                 'ids': self.pages[page].ids_get(),
@@ -1299,16 +1360,16 @@ class Main(object):
                 }
         tryton.plugins.execute(datas, self.window)
 
-    @staticmethod
-    def sig_quit(widget=None):
+    @classmethod
+    def sig_quit(cls, widget=None):
+        rpc.logout()
         CONFIG['client.default_width'] = Main.get_main()._width
         CONFIG['client.default_height'] = Main.get_main()._height
         CONFIG.save()
         if hasattr(gtk, 'accel_map_save'):
             gtk.accel_map_save(os.path.join(get_config_dir(), 'accel.map'))
 
-        if gtk.main_level() > 0:
-            gtk.main_quit()
+        cls.tryton_client.quit_mainloop()
 
     def sig_close(self, widget):
         if common.sur(_("Do you really want to quit?"), parent=self.window):
@@ -1332,15 +1393,37 @@ class Main(object):
             self._height = int(event.height)
         return False
 
+    def sig_window_state(self, widget, event):
+        CONFIG['client.maximize'] = (event.new_window_state ==
+                gtk.gdk.WINDOW_STATE_MAXIMIZED)
+        return False
+
     def win_add(self, page):
+        if page.model == 'ir.ui.menu' and not self.pane.get_child1():
+            screen = page.screen
+            screen.screen_container.alternate_view = True
+            screen.switch_view(view_type=screen.current_view.view_type)
+            self.pane.pack1(screen.screen_container.alternate_viewport)
+            self.menu_screen = screen
+            screen.display(set_cursor=True)
+            return
+        elif page.model == 'ir.ui.menu':
+            self.sig_win_menu()
         previous_page_id = self.notebook.get_current_page()
         previous_widget = self.notebook.get_nth_page(previous_page_id)
         self.previous_pages[page] = previous_widget
         self.pages.append(page)
-        hbox = gtk.HBox()
+        hbox = gtk.HBox(spacing=3)
+        icon_w, icon_h = gtk.icon_size_lookup(gtk.ICON_SIZE_SMALL_TOOLBAR)
+        if page.icon is not None:
+            common.ICONFACTORY.register_icon(page.icon)
+            image = gtk.Image()
+            image.set_from_stock(page.icon, gtk.ICON_SIZE_SMALL_TOOLBAR)
+            hbox.pack_start(image, expand=False, fill=False)
+            noise_size = 2 * icon_w + 3
+        else:
+            noise_size = icon_w + 3
         name = page.name
-        if page.model == 'ir.ui.menu':
-            name = _('Menu')
         label = gtk.Label(name)
         self.tooltips.set_tip(label, page.name)
         self.tooltips.enable()
@@ -1348,8 +1431,7 @@ class Main(object):
         hbox.pack_start(label, expand=True, fill=True)
         layout = label.get_layout()
         w, h = layout.get_size()
-        icon_w, icon_h = gtk.icon_size_lookup(gtk.ICON_SIZE_SMALL_TOOLBAR)
-        if (w / pango.SCALE) > 120 - icon_w:
+        if (w / pango.SCALE) > 120 - noise_size:
             label2 = gtk.Label('...')
             self.tooltips.set_tip(label2, page.name)
             hbox.pack_start(label2, expand=False, fill=False)
@@ -1383,10 +1465,8 @@ class Main(object):
 
     def sb_set(self, view=None):
         if not view:
-            view = self._wid_get()
+            view = self.get_page()
         for i in self.buttons:
-            if i in ('but_menu', 'but_request'):
-                continue
             if self.buttons[i]:
                 self.buttons[i].set_sensitive(
                         bool(view and (i in view.handlers)))
@@ -1449,10 +1529,13 @@ class Main(object):
             del page
 
             self.notebook.set_current_page(next_page_id)
+        if not self.pages and self.menu_screen:
+            self.menu_screen.current_view.set_cursor()
         return self.notebook.get_current_page() != -1
 
-    def _wid_get(self):
-        page_id = self.notebook.get_current_page()
+    def get_page(self, page_id=None):
+        if page_id is None:
+            page_id = self.notebook.get_current_page()
         if page_id == -1:
             return None
         page_widget = self.notebook.get_nth_page(page_id)
@@ -1462,7 +1545,7 @@ class Main(object):
         return None
 
     def _sig_child_call(self, widget, button_name):
-        wid = self._wid_get()
+        wid = self.get_page()
         if wid:
             res = True
             if button_name in wid.handlers:
@@ -1473,12 +1556,21 @@ class Main(object):
     def _sig_page_changt(self, notebook, page, page_num):
         self.last_page = self.current_page
         self.current_page = self.notebook.get_current_page()
+
+        current_form = self.get_page(self.current_page)
+        parent = self.toolbar.get_parent()
+        if parent:
+            parent.remove(self.toolbar)
+        current_form.toolbar_box.add(self.toolbar)
+        current_form.toolbar_box.show_all()
+
         self.sb_set()
 
     def sig_db_new(self, widget):
         if not self.sig_logout(widget):
             return False
-        dia = DBCreate(self.sig_login)
+        dia = DBCreate(CONFIG['login.server'], int(CONFIG['login.port']),
+            sig_login=self.sig_login)
         res = dia.run(self.window)
         if res:
             CONFIG.save()
@@ -1616,3 +1708,124 @@ class Main(object):
         else:
             rpc.logout()
             Main.get_main().refresh_ssl()
+
+    def _open_url(self, url):
+        url = urllib.unquote(url)
+        urlp = urlparse(url)
+        if not urlp.scheme == 'tryton':
+            return
+        urlp = urlparse('http' + url[6:])
+        hostname, port = (urlp.netloc.split(':', 1)
+                + [CONFIG.defaults['login.port']])[:2]
+        database, path = (urlp.path[1:].split('/', 1) + [None])[:2]
+        if (not path or
+                hostname != rpc._SOCK.hostname or
+                int(port) != rpc._SOCK.port or
+                database != rpc._DATABASE):
+            return
+        type_, path = (path.split('/', 1) + [''])[:2]
+        params = {}
+        if urlp.params:
+            try:
+                params = dict(param.split('=', 1)
+                        for param in urlp.params.split('&'))
+            except Exception:
+                return
+
+        def open_model(path):
+            model, path = (path.split('/', 1) + [''])[:2]
+            if not model:
+                return
+            res_id = False
+            mode = None
+            try:
+                view_ids = json.loads(params.get('views', 'false'))
+                limit = json.loads(params.get('limit', 'null'))
+                auto_refresh = json.loads(params.get('auto_refresh', 'false'))
+                name = json.loads(params.get('window_name', 'false'))
+                search_value = json.loads(params.get('search_value', '{}'))
+                domain = json.loads(params.get('domain', '[]'))
+                context = json.loads(params.get('context', '{}'))
+            except ValueError:
+                return
+            if path:
+                try:
+                    res_id = int(path)
+                except ValueError:
+                    return
+                mode = ['form', 'tree']
+            try:
+                Window.create(view_ids, model, res_id=res_id, domain=domain,
+                        window=self.window, context=context, mode=mode,
+                        name=name, limit=limit, auto_refresh=auto_refresh,
+                        search_value=search_value)
+            except Exception:
+                return
+
+        def open_wizard(wizard):
+            if not wizard:
+                return
+            try:
+                data = json.loads(params.get('data', '{}'))
+                direct_print = json.loads(params.get('direct_print', 'false'))
+                email_print = json.loads(params.get('email_print', 'false'))
+                email = json.loads(params.get('email', 'null'))
+                name = json.loads(params.get('name', 'false'))
+                window = json.loads(params.get('window', 'false'))
+                context = json.loads(params.get('context', '{}'))
+            except ValueError:
+                return
+            try:
+                if window:
+                    Window.create_wizard(wizard, data, self.window,
+                            direct_print=direct_print, email_print=email_print,
+                            email=email, name=name, context=context)
+                else:
+                    Wizard.execute(wizard, data, self.window,
+                            direct_print=direct_print, email_print=email_print,
+                            email=email, context=context)
+            except Exception:
+                return
+
+        def open_report(report):
+            if not report:
+                return
+            try:
+                data = json.loads(params.get('data'))
+                direct_print = json.loads(params.get('direct_print', 'false'))
+                email_print = json.loads(params.get('email_print', 'false'))
+                email = json.loads(params.get('email', 'null'))
+                name = json.loads(params.get('name', 'false'))
+                context = json.loads(params.get('context', '{}'))
+            except ValueError:
+                return
+            try:
+                Action.exec_report(report, data, self.window,
+                        direct_print=direct_print, email_print=email_print,
+                        email=email, context=context)
+            except Exception:
+                return
+
+        def open_url():
+            try:
+                url = json.loads(params.get('url', 'false'))
+            except ValueError:
+                return
+            if url:
+                webbrowser.open(url, new=2)
+
+        if type_ == 'model':
+            open_model(path)
+        elif type_ == 'wizard':
+            open_wizard(path)
+        elif type_ == 'report':
+            open_report(path)
+        elif type_ == 'url':
+            open_url()
+
+    def open_url(self, url):
+        def idle_open_url():
+            with gtk.gdk.lock:
+                self._open_url(url)
+                return False
+        gobject.idle_add(idle_open_url)
