@@ -28,6 +28,7 @@ class Button(object):
         if attrs.get('icon', False):
             try:
                 stock = attrs['icon']
+                common.ICONFACTORY.register_icon(stock)
                 icon = gtk.Image()
                 icon.set_from_stock(stock, gtk.ICON_SIZE_SMALL_TOOLBAR)
                 self.widget.set_image(icon)
@@ -82,7 +83,7 @@ class Button(object):
                             }, self.form.window, context=ctx)
                 else:
                     raise Exception('Unallowed button type')
-                self.form.screen.reload(writen=True)
+                self.form.screen.reload(written=True)
         else:
             self.form.screen.display()
 
@@ -101,6 +102,7 @@ class Button(object):
             stock = state_changes['icon']
             if stock:
                 try:
+                    common.ICONFACTORY.register_icon(stock)
                     icon = gtk.Image()
                     icon.set_from_stock(stock, gtk.ICON_SIZE_SMALL_TOOLBAR)
                     self.widget.set_image(icon)
@@ -290,9 +292,10 @@ class _container(object):
 
 class ParserForm(ParserInterface):
 
-    def __init__(self, window, parent=None, attrs=None, screen=None):
+    def __init__(self, window, parent=None, attrs=None, screen=None,
+            children_field=None):
         super(ParserForm, self).__init__(window, parent=parent, attrs=attrs,
-                screen=screen)
+                screen=screen, children_field=children_field)
         self.widget_id = 0
 
     def parse(self, model_name, root_node, fields, notebook=None, paned=None,
@@ -320,10 +323,10 @@ class ParserForm(ParserInterface):
             attrs = common.node_attributes(node)
             if not cursor_widget:
                 if attrs.get('name') and fields.get(attrs['name']) \
-                        and not fields[attrs['name']].attrs.get('exclude_field',
-                                False):
+                        and attrs['name'] != self.screen.exclude_field:
                     cursor_widget = attrs.get('name')
             if node.localName == 'image':
+                common.ICONFACTORY.register_icon(attrs['name'])
                 icon = Image(attrs)
                 button_list.append(icon)
                 icon.set_from_stock(attrs['name'], gtk.ICON_SIZE_DIALOG)
@@ -356,6 +359,9 @@ class ParserForm(ParserInterface):
             elif node.localName == 'label':
                 text = attrs.get('string', '')
                 if 'name' in attrs and attrs['name'] in fields:
+                    if attrs['name'] == self.screen.exclude_field:
+                        container.empty_add(int(attrs.get('colspan', 1)))
+                        continue
                     for attr_name in ('states', 'invisible'):
                         if attr_name not in attrs \
                                 and attr_name in fields[attrs['name']].attrs:
@@ -462,12 +468,23 @@ class ParserForm(ParserInterface):
             elif node.localName == 'page':
                 if CONFIG['client.form_tab'] == 'left':
                     angle = 90
+                    tab_box  = gtk.VBox(spacing=3)
+                    image_pos, image_rotate = ('end',
+                        gtk.gdk.PIXBUF_ROTATE_COUNTERCLOCKWISE)
                 elif CONFIG['client.form_tab'] == 'right':
                     angle = -90
+                    tab_box = gtk.VBox(spacing=3)
+                    image_pos, image_rotate = ('start',
+                        gtk.gdk.PIXBUF_ROTATE_CLOCKWISE)
                 else:
                     angle = 0
+                    tab_box = gtk.HBox(spacing=3)
+                    image_pos, image_rotate = ('start',
+                        gtk.gdk.PIXBUF_ROTATE_NONE)
                 text = attrs.get('string', '')
                 if 'name' in attrs and attrs['name'] in fields:
+                    if attrs['name'] == self.screen.exclude_field:
+                        continue
                     for attr_name in ('states', 'invisible'):
                         if attr_name in fields[attrs['name']].attrs:
                             attrs[attr_name] = \
@@ -478,9 +495,22 @@ class ParserForm(ParserInterface):
                     text = _('No String Attr.')
                 if '_' not in text:
                     text = '_' + text
-                label = gtk.Label(text)
-                label.set_angle(angle)
-                label.set_use_underline(True)
+                tab_label = gtk.Label(text)
+                tab_label.set_angle(angle)
+                tab_label.set_use_underline(True)
+                tab_box.pack_start(tab_label)
+                if 'icon' in attrs:
+                    common.ICONFACTORY.register_icon(attrs['icon'])
+                    pixbuf = tab_box.render_icon(attrs['icon'],
+                        gtk.ICON_SIZE_SMALL_TOOLBAR)
+                    pixbuf = pixbuf.rotate_simple(image_rotate)
+                    icon = gtk.Image()
+                    icon.set_from_pixbuf(pixbuf)
+                    if image_pos == 'end':
+                        tab_box.pack_end(icon)
+                    else:
+                        tab_box.pack_start(icon)
+                tab_box.show_all()
                 widget, widgets, buttons, spam, notebook_list2, cursor_widget2 = \
                         self.parse(model_name, node, fields, notebook,
                                 tooltips=tooltips)
@@ -503,10 +533,13 @@ class ParserForm(ParserInterface):
                 scrolledwindow.add(viewport)
 
                 button_list.append(scrolledwindow)
-                notebook.append_page(scrolledwindow, label)
+                notebook.append_page(scrolledwindow, tab_box)
 
             elif node.localName == 'field':
                 name = str(attrs['name'])
+                if name == self.screen.exclude_field:
+                    container.empty_add(int(attrs.get('colspan', 1)))
+                    continue
                 if name not in fields:
                     container.empty_add(int(attrs.get('colspan', 1)))
                     log = logging.getLogger('view')
@@ -518,7 +551,8 @@ class ParserForm(ParserInterface):
                     continue
                 for attr_name in ('relation', 'domain', 'selection',
                         'relation_field', 'string', 'views', 'invisible',
-                        'add_remove', 'sort', 'context', 'size'):
+                        'add_remove', 'sort', 'context', 'size', 'filename',
+                        'autocomplete'):
                     if attr_name in fields[name].attrs and \
                             not attr_name in attrs:
                         attrs[attr_name] = fields[name].attrs[attr_name]
@@ -567,6 +601,9 @@ class ParserForm(ParserInterface):
                 button_list += buttons
                 text = ''
                 if 'name' in attrs and attrs['name'] in fields:
+                    if attrs['name'] == self.screen.exclude_field:
+                        container.empty_add(int(attrs.get('colspan', 1)))
+                        continue
                     for attr_name in ('states', 'invisible'):
                         if attr_name in fields[attrs['name']].attrs:
                             attrs[attr_name] = fields[attrs['name']
@@ -827,6 +864,7 @@ from many2one import Many2One
 from url import Email, URL, CallTo, SIP
 from image import Image as Image2
 from progressbar import ProgressBar
+from one2one import One2One
 
 
 WIDGETS_TYPE = {
@@ -853,4 +891,5 @@ WIDGETS_TYPE = {
     'sip' : (SIP, 1, False, False),
     'image' : (Image2, 1, False, False),
     'progressbar': (ProgressBar, 1, False, False),
+    'one2one': (One2One, 1, False, False),
 }
