@@ -32,12 +32,12 @@ class WinSearch(object):
         self.accel_group = gtk.AccelGroup()
         self.win.add_accel_group(self.accel_group)
 
-        self.but_find = self.win.add_button(gtk.STOCK_FIND, gtk.RESPONSE_APPLY)
-        self.but_new = self.win.add_button(gtk.STOCK_NEW, gtk.RESPONSE_ACCEPT)
         self.but_cancel = self.win.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
+        self.but_find = self.win.add_button(gtk.STOCK_FIND, gtk.RESPONSE_APPLY)
         self.but_ok = self.win.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)
         self.but_ok.add_accelerator('clicked', self.accel_group,
                 gtk.keysyms.Return, gtk.gdk.CONTROL_MASK, gtk.ACCEL_VISIBLE)
+        self.but_new = self.win.add_button(gtk.STOCK_NEW, gtk.RESPONSE_ACCEPT)
 
         hbox = gtk.HBox()
         hbox.show()
@@ -48,7 +48,7 @@ class WinSearch(object):
         self.win.vbox.pack_start(scrollwindow, expand=True, fill=True)
 
         self.screen = Screen(model, self.win, domain=domain,
-                view_type=['tree'], context=context,
+                mode=['tree'], context=context,
                 views_preload=views_preload, row_activate=self.sig_activate)
         self.view = self.screen.current_view
         self.view.unset_editable()
@@ -104,14 +104,11 @@ class WinSearch(object):
             elif button == gtk.RESPONSE_ACCEPT:
                 res = None
                 screen = Screen(self.model_name, self.win, domain=self.domain,
-                        context=self.context, view_type=['form'])
+                        context=self.context, mode=['form'])
                 win = WinForm(screen, self.win, new=True)
-                while win.run():
+                if win.run():
                     if screen.save_current():
                         res = [screen.current_record.id]
-                        break
-                    else:
-                        screen.display()
                 win.destroy()
                 end = True
             else:
